<?php
require_once('cmd_main.php');


/**
 * Database maintenance procedure.
 */
class process_DBCleanup {

    private $config;

    public function __construct(lib_Config $config) {
         $this->config = $config;
    }

    private function echoStatus($res, $type) {
        if ($res < 0) {
            echo "Can not purge $type.\n";
        } else {
            echo ($res == 0 ? 'No' : $res)." $type purged.\n";
        }
    }

    public function run() {
        $dbHandle = lib_Db::get();
        $this->echoStatus(self::removeExpiredSessions($dbHandle), 'sessions');
        $this->echoStatus(self::removeDeletedRecords($dbHandle), 'records');
        $dbHandle = null;
    }

    protected function removeExpiredSessions(lib_Db $dbHandle) {
        $sql = 'delete from idp_session where (last_visit + expire_time) < ' . time();
        $result = $dbHandle->exec($sql);
        lib_Config::getLog()->info("Removed $result stale idp sessions.");
        return $result;
    }

    protected function removeDeletedRecords(lib_Db $dbHandle) {
        // This period determines how long we keep deleted records.
        $retention_period = $this->config->keep_deleted_records_for;

        /*
         * TODO (refactoring, low) It would be nice to obtain this list automatically somehow
         * (either from some registry or from DB metadata): we need all tables with deferred
         * deletion (ones that have 'delete_time' field).
         */
        $tables = array(
            'sp',
            'sp_user',
            'federation',
            'federation_item',
            'delegation' );
        $cutTime = new DateTime('-' . $retention_period);
        $result = 0;
        foreach ($tables as $tableName) {
            $result += self::purgeDeletedRecords($dbHandle, $tableName, $cutTime);
        }
        return $result;
    }

    protected function purgeDeletedRecords(lib_Db $dbHandle, $tableName, $cutTime) {
        $sql = "delete from $tableName where delete_time < " . $cutTime->format('U');
        $result = $dbHandle->exec($sql);
        lib_Config::getLog()->info(
                "Removed $result records"
                . " deleted before {$cutTime->format('Y-m-d\TH:i:s\Z')} from table $tableName." );
        return $result;
    }
}


function runDBCleanup($config) {
    $worker = new process_DBCleanup($config);
    $worker->run();
}


cmd_main('DB Cleanup', 'runDBCleanup');


?>